package com.vendor.burn.commands;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.vendor.burn.commands.SerialCommand;
import com.vendor.burn.model.config.BurnConfig;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.utilitily.CRC16;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.CRC32;

public class CrcCommand extends SerialCommand {
  private static final String KEY_CRC32 = "crc32";
  
  private static final String KEY_FLAG = "==>";
  
  private static int defaultTftpPort = 69;
  
  private CrcCommand(String command) {
    setCommand(command);
  }
  
  public static com.vendor.burn.commands.CrcCommand sendCRCCommand(File file, String address) {
    long fileSize = file.length();
    StringBuilder commandStr = new StringBuilder();
    commandStr.append("crc32");
    commandStr.append(" ");
    commandStr.append(address);
    commandStr.append(" ");
    commandStr.append(Long.toHexString(fileSize));
    com.vendor.burn.commands.CrcCommand crcCommand = new com.vendor.burn.commands.CrcCommand(commandStr.toString());
    crcCommand.setTimeout(calculateTimeout(file.length()));
    return crcCommand;
  }
  
  public static int getTFTPPort() {
    int port;
    if (ToolUtils.isCmdLine()) {
      port = BurnConfig.getPort();
    } else {
      port = defaultTftpPort;
    } 
    return port;
  }
  
  private static int calculateTimeout(long length) {
    int speed = 0;
    if (ToolUtils.isCmdLine()) {
      speed = PreferenceConfig.getTftpSpeed();
    } else {
      speed = FastbootPreferences.getInt("vendor.burn.tftp.speed");
    } 
    if (speed == 0)
      speed = 1; 
    return (int)((length / speed + 2L) * 1000L);
  }
  
  public static int calculateCheckSum(File file) throws IOException {
    int checkSum = 0;
    InputStream inputStream = null;
    try {
      inputStream = new BufferedInputStream(new FileInputStream(file.getPath()));
      byte[] tempBytes = new byte[32];
      while (inputStream.read(tempBytes) != -1) {
        for (int i = 0; i < tempBytes.length; i++)
          checkSum += tempBytes[i] & 0xFF; 
      } 
    } catch (IOException e) {
      return 0;
    } finally {
      try {
        if (inputStream != null)
          inputStream.close(); 
      } catch (IOException iOException) {}
    } 
    return checkSum;
  }
  
  public static int calculateCRC16(File file) throws IOException {
    int crc16 = 65535;
    InputStream inputStream = null;
    try {
      inputStream = new BufferedInputStream(new FileInputStream(file.getPath()));
      byte[] tempBytes = new byte[32];
      while (inputStream.read(tempBytes) != -1) {
        int[] dataList = new int[8];
        byte[] bytes = null;
        for (int k = 0; k < dataList.length; k++) {
          bytes = new byte[4];
          for (int i = 0; i < 4; i++)
            bytes[i] = tempBytes[4 * k + 3 - i]; 
          dataList[k] = CRC16.bytesToInt(bytes);
        } 
        crc16 = CRC16.CRC_CAL_4B(crc16, 1, dataList);
      } 
    } catch (IOException e) {
      return 0;
    } finally {
      try {
        if (inputStream != null)
          inputStream.close(); 
      } catch (IOException iOException) {}
    } 
    return crc16 & 0xFFFF;
  }
  
  public static long calculateCRC(File file) throws IOException {
    CRC32 crc32 = new CRC32();
    InputStream inputStream = null;
    try {
      inputStream = new BufferedInputStream(new FileInputStream(file.getPath()));
      int read = 0;
      while ((read = inputStream.read()) != -1)
        crc32.update(read); 
    } catch (IOException e) {
      return 0L;
    } finally {
      try {
        if (inputStream != null)
          inputStream.close(); 
      } catch (IOException iOException) {}
    } 
    return crc32.getValue();
  }
  
  public static long receiveCRC(String crcStr) {
    int flag = crcStr.indexOf("==>");
    int endIndex = crcStr.indexOf("\r\n[EOT]");
    if (flag != -1 && endIndex != -1 && flag + "==>".length() < endIndex) {
      String substring = crcStr.substring(flag + "==>".length(), endIndex).trim();
      return Long.parseLong(substring, 16);
    } 
    return 0L;
  }
  
  public static boolean compareFileCRC(long fileCrc, long receiveCrcValue) {
    if (fileCrc == receiveCrcValue)
      return true; 
    return false;
  }
}
